require 'consumerable/version'

module Consumerable
  extend ActiveSupport::Autoload

  class << self
    attr_accessor :configuration
  end

  def self.log(message)
    if configuration && configuration.perform_logging?
      configuration.logger.info("CONSUMERABLE: #{message}")
    end
  end

  def self.configure
    self.configuration = Consumerable::Configuration.new
    yield self.configuration
    self.configuration
  end

  def self.content_type_string(content_type)
    {
      json: 'application/json',
      xml: 'application/xml'
    }.fetch(content_type)
  end

  eager_autoload do
    autoload :Associations
    autoload :Configuration
    autoload :Connection
    autoload :Creatable
    autoload :Deletable
    autoload :Listable
    autoload :Model
  end

  def self.eager_autoload!
    super
    Consumerable::Associations.eager_load!
  end
end
