module Consumerable
  module Associations
    class HasManyProxy < BasicObject

      attr_reader :parent, :target_klass, :target, :source

      def initialize(parent, target_klass, target, source)
        @parent         = parent
        @target_klass   = target_klass
        @target         = target
        @source         = source
      end

      def method_missing(name, *args, &block)
        target.send(name, *args, &block)
      end

      def create(attributes = {})
        build(attributes).save
      end

      def build(attributes)
        target_klass.new(attributes).tap do |item|
          item.send("#{source}=", parent)
          target << item
        end
      end

      def tap
        yield self
        self
      end
    end
  end
end
