module Consumerable
  module Creatable
    extend ActiveSupport::Concern

    module ClassMethods
      def create(attributes = {})
        new(attributes).tap(&:save)
      end

      def create!(attributes = {})
        new(attributes).tap(&:save!)
      end
    end

    def save
      if valid?
        create_record
      else
        false
      end
    end

    def save!
      if valid?
        create_record
      else
        raise Consumerable::Errors::RecordInvalid.new(self)
      end
    end

    private

    def create_record
      self.attributes =
        Consumerable::Connection.post(
          _inject_path_params(create_path), attributes_for_api.except(:id).
          delete_if { |k, v| v.blank? }
        )
    end
  end
end
